<?php

namespace App\Controllers;

use App\Models\GitModel;
use CodeIgniter\HTTP\RequestInterface;
use CodeIgniter\HTTP\ResponseInterface;
use Psr\Log\LoggerInterface;

class Inscription extends PortalController {

	/**
	 * Constructor.
	 *
	 * @param RequestInterface  $request
	 * @param ResponseInterface $response
	 * @param LoggerInterface   $logger
	 */
	public function initController(RequestInterface $request, ResponseInterface $response, LoggerInterface $logger) {
        error_reporting(E_ALL);
        ini_set("display_errors", 1);

		parent::initController($request, $response, $logger);
	}

	public function index() {
		$data = $this->getMinimalData();

		$piratePortal = new \Config\PiratePortal();

		$data["reductPrice"] = $piratePortal->reductPrice;
		$data["fullPrice"] = $piratePortal->fullPrice;
		$data["taxCreditPercentage"] = $piratePortal->taxCreditPercentage;

		return $this->fullView('Inscription/index', $data);
	}

	public function observer() {
		$data = $this->getMinimalData();
		$data["price"] = "observer";

		$session = session();
		$session->set("inscription", null);

		return $this->fullView('Inscription/inscription_step_one', $data);
	}

	public function member() {
		$data = $this->getMinimalData();

		$session = session();
		$session->set("inscription", null);

		$validation =  \Config\Services::validation();
		$validation->setRule('price', 'price', 'required');

		if ($validation->withRequest($this->request)->run()) {
			$data["price"] = $this->request->getVar("price");

			return $this->fullView('Inscription/inscription_step_one', $data);
		}

		return redirect()->to('/inscription');
	}

	public function activate($email = "", $code = "") {
		$data = $this->getMinimalData();

		$galetteDB = db_connect('galette');

		$userModel = new \App\Models\UserModel($galetteDB);

		$user = $userModel->findByMail($email);

		if ($user && $user->use_activation_key == $code) {
			$user->use_activated = 1;

			$userModel->save($user);

			return $this->fullView('Inscription/activated', $data);
		}

		return $this->fullView('Inscription/noActivation', $data);
	}

	public function doFirstStep() {
		$data = $this->getMinimalData();

		$validation =  \Config\Services::validation();
		$validation->setRule('price', 'price', 'required');
		$validation->setRule('nickname', 'nickname', 'required');
		$validation->setRule('email', 'email', 'required|valid_email');
		$validation->setRule('password', 'password', 'required');
		$validation->setRule('rgpd', 'rgpd', 'required');

		$data["price"] = $this->request->getVar("price");
		$data["nickname"] = $this->request->getVar("nickname");
		$data["email"] = $this->request->getVar("email");

		$data["validation"] = $validation;

		if ($validation->withRequest($this->request)->run()) {

			$session = session();
			
			$inscription = [
				"price" => $data["price"],
				"nickname" => $data["nickname"],
				"email" => $data["email"],
				"password" => $this->request->getVar("password"),
			];
			$session->set("inscription", $inscription);

			if ($data["price"] == "observer") {
				$galetteDB = db_connect('galette');

				$userModel = new \App\Models\UserModel($galetteDB);
				$user = $userModel->findByMail($inscription["email"]);

				if ($user) {
					// hum
				}
				else {
					$user = new \App\Entities\User();

					$user->use_activation_key = hash("sha256", random_int(PHP_INT_MIN, PHP_INT_MAX) . time(), false);
					$user->use_activated = 0;
					$user->id_statut = \App\Entities\User::IS_OBSERVER;
					$user->pseudo_adh = $inscription["nickname"];
					$user->email_adh = $inscription["email"];
					$user->setPassword($inscription["password"]);

					$existingUser = $userModel->findByLogin($inscription["nickname"]);

					$user->login_adh = $inscription["nickname"];

					$userModel->save($user);

					$activationLink = site_url("inscription/activate/" . urlencode($user->email_adh) . "/" . $user->use_activation_key);

					// TODO send mail
					$email = \Config\Services::email();
					$email->setTo($inscription["email"]);

					$email->setSubject(lang("Inscription.observer.email.subject"));
					$email->setMessage(lang("Inscription.observer.email.message", [$activationLink]));
					
					$email->send();
				}

				return redirect()->to('/inscription/complete/observer');
//				return $this->fullView('Inscription/inscription_step_two', $data);
			}
			else {
				return $this->fullView('Inscription/inscription_step_two', $data);
			}
		}

		return $this->fullView('Inscription/inscription_step_one', $data);
	}

	public function doSecondStep() {
		$data = $this->getMinimalData();

		$validation =  \Config\Services::validation();
		$validation->setRule('price', 'price', 'required');

		$data["price"] = $this->request->getVar("price");

		$piratePortal = new \Config\PiratePortal();

		$data["reductPrice"] = $piratePortal->reductPrice;
		$data["fullPrice"] = $piratePortal->fullPrice;
		$data["taxCreditPercentage"] = $piratePortal->taxCreditPercentage;

		$data["firstname"]		= $this->request->getVar("firstname");
		$data["lastname"]		= $this->request->getVar("lastname");
		$data["nationality-iso"]= $this->request->getVar("nationality");
		$data["nationality"]	= $this->request->getVar("nationality-label");
		$data["address"]		= $this->request->getVar("address");
		$data["zipcode"]		= $this->request->getVar("zipcode");
		$data["city"]			= $this->request->getVar("city");
		$data["country"]		= $this->request->getVar("country");
		$data["telephone"]		= $this->request->getVar("telephone");

		$amount = $piratePortal->{$data["price"] . "Price"};
		$maximumAmount = $piratePortal->maximumAmount - $amount;

		$data["maximumAmount"] = $maximumAmount;

		$data["validation"] = $validation;

		if ($validation->withRequest($this->request)->run()) {
			$session = session();
		
			$inscription = $session->get("inscription");

			$inscription["firstname"]		= $data["firstname"];
			$inscription["lastname"]		= $data["lastname"];
			$inscription["nationality"] 	= $data["nationality"];
			$inscription["nationality-iso"] = $data["nationality-iso"];
			$inscription["address"] 		= $data["address"];
			$inscription["zipcode"] 		= $data["zipcode"];
			$inscription["city"]			= $data["city"];
			$inscription["country"] 		= $data["country"];
			$inscription["telephone"] 		= $data["telephone"];

			$session->set("inscription", $inscription);

			return $this->fullView('Inscription/inscription_step_three', $data);
		}

		return $this->fullView('Inscription/inscription_step_two', $data);
	}
	
	public function doThirdStep() {
		$data = $this->getMinimalData();

		$piratePortal = new \Config\PiratePortal();

		$data["reductPrice"] = $piratePortal->reductPrice;
		$data["fullPrice"] = $piratePortal->fullPrice;
		$data["taxCreditPercentage"] = $piratePortal->taxCreditPercentage;
		$data["price"] = $this->request->getVar("price");

		$amount = $piratePortal->{$data["price"] . "Price"};
		$maximumAmount = $piratePortal->maximumAmount - $amount;

		$data["maximumAmount"] = $maximumAmount;

		$validation =  \Config\Services::validation();
		$validation->setRule("price", "price", "required");

		$data["moreAmount"] = $this->request->getVar("moreAmount");

		if ($data["moreAmount"]) {
			$validation->setRule("moreAmount", "moreAmount", "numeric|greater_than_equal_to[0]|less_than_equal_to[$maximumAmount]");
		}

		$data["validation"] = $validation;

		if ($validation->withRequest($this->request)->run()) {
			$session = session();
		
			$inscription = $session->get("inscription");

			$transactionDB = db_connect('transactions');
			$transactionModel = new \App\Models\TransactionModel($transactionDB);

			$transaction = new \App\Entities\Transaction();
			$transaction->tra_reference = "";
			$saved = $transactionModel->save($transaction);

			$transaction->tra_id = $transactionModel->getInsertID();;

			$purpose = array();

			$purpose["forumPseudo"] = $inscription["nickname"];
			$purpose["join"] = $amount;
			$purpose["donation"] = str_replace(",", ".", ($data["moreAmount"] ?? 0));


/*
if (isset($_REQUEST["localSection"]) && $_REQUEST["localSection"]) {
	$purpose["local"] = array();
	$purpose["local"]["section"] = $_REQUEST["localSection"];

	if (isset($_REQUEST["localDonation"]) && $_REQUEST["localDonation"]) {
		$purpose["local"]["donation"] = str_replace(",", ".", $_REQUEST["localDonation"]);
	}
}
if (isset($_REQUEST["forum"]) && isset($_REQUEST["pseudo"]) && $_REQUEST["pseudo"]) {
	$purpose["forumPseudo"] = $_REQUEST["pseudo"];
}
*/

/* TODO
if (isset($_REQUEST["subscription"])) {
	$purpose["reportSubscription"] = true;
}
*/
/*
if (isset($_REQUEST["comment"]) && $_REQUEST["comment"]) {
	$purpose["comment"] = $_REQUEST["comment"];
}
*/

/*
if (isset($_REQUEST["projectId"]) && $_REQUEST["projectId"]) {
	$purpose["project"] = array();

	$projectBo = ProjectBo::newInstance($connection);
	$project = $projectBo->getProject($_REQUEST["projectId"]);

	$purpose["project"]["code"] = $project["pro_code"];
	$purpose["project"]["donation"] = str_replace(",", ".", $_REQUEST["projectDonation"]);
	$purpose["project"]["additionalDonation"] = str_replace(",", ".", $_REQUEST["projectAdditionalDonation"]);
}
if (isset($_REQUEST["rejoin"]) && $_REQUEST["rejoin"] == "true") {
	$purpose["rejoin"] = true;
}
*/






























			// create reference
			$transaction->tra_reference = "" . $transaction->tra_id;
			while(strlen($transaction->tra_reference) < 8) {
				$transaction->tra_reference = "0" . $transaction->tra_reference;
			}
			$transaction->tra_reference = "PP" . $transaction->tra_reference;

			$transaction->tra_amount = $amount + ($data["moreAmount"] ?? 0);

			$transaction->tra_email = $inscription["email"];
			$transaction->tra_date = date("c");
			
			$transaction->tra_firstname = $inscription["firstname"];
			$transaction->tra_lastname = $inscription["lastname"];
			
			$transaction->tra_address = $inscription["address"];
			$transaction->tra_zipcode = $inscription["zipcode"];
			$transaction->tra_city = $inscription["city"];
			$transaction->tra_country = $inscription["country"];
			$transaction->tra_telephone = $inscription["telephone"];

			$transaction->tra_purpose = json_encode($purpose);

			$transaction->tra_nationality_iso = $inscription["nationality-iso"];
			$transaction->tra_nationality = $inscription["nationality"];

			$transactionModel->save($transaction);

			$data["paybox"] = new \Config\Paybox();
			$this->preparePaybox($data, $transaction);

			return $this->fullView('Inscription/goPayment', $data);
		}

		return $this->fullView('Inscription/inscription_step_three', $data);
	}

	public function complete($status) {
		$session = session();

		$data = $this->getMinimalData();
		$data["status"] = $status;
		$data["inscription"] = $session->inscription;

		return $this->fullView('Inscription/complete', $data);
	}
	
	protected function preparePaybox(&$data, $transaction) {
		// On renseigne les différents paramètres de la requête de paiement
		// Le montant en centime
		$data["PBX_TOTAL"] = $transaction->tra_amount * 100;
		// Devise en euro
		$data["PBX_DEVISE"] = $data["paybox"]->PBX_DEVISE;
		// Référence du paiement
		$data["PBX_CMD"] = $transaction->tra_reference;
		// L'email du donateur
		$data["PBX_PORTEUR"] = $transaction->tra_email;
		// Le format des données retour
		$data["PBX_RETOUR"] = "Mt:M;Ref:R;Auto:A;Erreur:E;Sign:K";
		// L'algorithme de hashage
		$data["PBX_HASH"] = "SHA512";
		// Date du paiement
		$data["PBX_TIME"] = $transaction->tra_date;

		$code = $data["PBX_CMD"] . $data["PBX_TOTAL"];
		$code = strtoupper(hash('md5', $code, false));

		$data["code"] = $code;

		$data["acceptedUrl"] = $data["paybox"]->acceptedUrl . "$code";
		$data["refusedUrl"] = $data["paybox"]->refusedUrl . "";
		$data["canceledUrl"] = $data["paybox"]->canceledUrl . "";
		$data["ipnUrl"] = $data["paybox"]->ipnUrl . "$code";

		// On crée la chaîne à hacher sans URLencodage ET ON LIMITE AUX CARTES BANCAIRES
		$msg =
		"PBX_SITE=" . $data["paybox"]->PBX_SITE .
		"&PBX_RANG=" . $data["paybox"]->PBX_RANG .
		"&PBX_IDENTIFIANT=" . $data["paybox"]->PBX_IDENTIFIANT .
		"&PBX_TOTAL=" . $data["PBX_TOTAL"] .
		"&PBX_DEVISE=" . $data["PBX_DEVISE"] .
		"&PBX_CMD=" . $data["PBX_CMD"] .
		"&PBX_PORTEUR=" . $data["PBX_PORTEUR"] .
		"&PBX_RETOUR=" . $data["PBX_RETOUR"] .
		"&PBX_HASH=" . $data["PBX_HASH"] .
		"&PBX_TIME=" . $data["PBX_TIME"] .
		"&PBX_TYPEPAIEMENT=CARTE" .
		"&PBX_TYPECARTE=CB" .
		"&PBX_EFFECTUE=" . urlencode($data["acceptedUrl"]) . "" .
		"&PBX_REFUSE=" . urlencode($data["refusedUrl"]) . "" .
		"&PBX_ANNULE=" . urlencode($data["canceledUrl"]) . "" .
		"&PBX_REPONDRE_A=" . urlencode($data["ipnUrl"]) . "" .
		"&PBX_3DS=N".
		"";

		// Clé de hashahge en ASCII, On la transforme en binaire
		$binKey = pack("H*", $data["paybox"]->secretKey);
		// On calcule l’empreinte (à renseigner dans le paramètre PBX_HMAC) grâce à la fonction hash_hmac et la clé binaire
		// On envoie via la variable PBX_HASH l'algorithme de hachage qui a été utilisé (SHA512 dans ce cas)
		$data["hmac"] = strtoupper(hash_hmac('sha512', $msg, $binKey));
	}
}