<?php

namespace App\Controllers;

use App\Models\GitModel;
use CodeIgniter\HTTP\RequestInterface;
use CodeIgniter\HTTP\ResponseInterface;
use Psr\Log\LoggerInterface;

class Program extends SearchController {
	private $gitModel = null;

	/**
	 * Constructor.
	 *
	 * @param RequestInterface  $request
	 * @param ResponseInterface $response
	 * @param LoggerInterface   $logger
	 */
	public function initController(RequestInterface $request, ResponseInterface $response, LoggerInterface $logger) {
        error_reporting(E_ALL);
        ini_set("display_errors", 1);

		parent::initController($request, $response, $logger);

		$this->gitModel = new GitModel();
		$this->gitModel->repository = "https://sources.partipirate.org/partipirate/programme/-/tree/master"; // TODO get it from config
	}
	
	public function index($path = "", $format = "html") {
		$path = str_replace("___", "/", $path);

		$cache = \Config\Services::cache();

		$cacheKey = GitModel::getProgramCacheKey($path);

		$contentCache = $cache->get($cacheKey);
		if (!$contentCache) {
			$program = $this->gitModel->getProgramEntry($path);
	
			if (!$program) {
				return $this->getProgramEntries($path . ($path ? "/" : ""), $format);
			}

			$parentPath = explode("/", $path);
			unset($parentPath[count($parentPath) - 1]);
			$parentPath = implode("/", $parentPath) . "/";

			$parentProgramCategories = $this->getProgramEntries($parentPath, "raw");

			$program["parent_path"] = substr($parentPath, 0, -1);
			$program["parent"] = $parentProgramCategories;

			$cache->save($cacheKey, json_encode($program), 600);
			$program["from_cache"] = false;
		}
		else {
			$program = json_decode($contentCache, true);
			$program["from_cache"] = true;

			if (is_array($program) && isset($program["entries"])) {
				return $this->getProgramEntries($path . ($path ? "/" : ""), $format);
			}
		}

		$program["cache_key"] = $cacheKey;

		// Renderisation d'un point programme

		switch($format) {
			case "json":
				return $this->response->setJSON($program);
				break;
			case "html":
			default:
				// load the view

				return $this->fullView('Program/programEntry', array("program" => $program, "path" => $path, "title" => "Parti Pirate - " . $program["TITRE"]));
				break;
		}
	}

	public function tags() {
		$programPoints = $this->getAllProgramPoints();

		$tags = [];

		foreach($programPoints as $point) {
			$entryTags = $point["TAGS"];

			foreach($entryTags as $tag) {
				if (!isset($tags[$tag])) {
					$tags[$tag] = [];
				}

				$tags[$tag][] = $point;
			}
		}

		ksort($tags, SORT_LOCALE_STRING);
		
		foreach($tags as $key => $entries) {
			usort($entries, array('App\Models\GitModel', "sortByTitle"));
			$tags[$key] = $entries;
		}
		
		return $this->fullView('Program/programTags', array("tags" => $tags, "title" => "Parti Pirate - Programme"));
	}

	public function search() {
//		$validation =  \Config\Services::validation();
//		$validation->setRule('price', 'price', 'required');

//		if ($validation->withRequest($this->request)->run()) {

		$search = [];
		$search["search"] = $this->request->getVar("search");

		$programPoints = $this->getAllProgramPoints();

		foreach($programPoints as $index => $programPoint) {
			$programPoints[$index]["scoring"] = $this->score($programPoint, $search);
//			$programPoint["scoring"] = [];
		}

		// normalize
		
		$programPoints = $this->normalizeScores($programPoints);
		
		// order

		usort($programPoints, array('App\Controllers\SearchController', "sortByScore"));

		return $this->fullView('Program/programList', array("category" => [], "programEntries" => $programPoints, "path" => "", "title" => "Parti Pirate - Programme - Recherche", "search" => $search));
//		}		
	}

	protected function getAllProgramPoints($path = "") {
		$cache = \Config\Services::cache();

		$cacheKey = "program." . md5($path);

//		echo "$path <br>";

		$contentCache = $cache->get($cacheKey);
		if (!$contentCache) {
			$program = $this->gitModel->getProgramEntry(substr($path, 0, -1));

			if ($program) {
				$parentPath = explode("/", $path);
				unset($parentPath[count($parentPath) - 1]);
				$parentPath = implode("/", $parentPath) . "/";
	
				$parentProgramCategories = $this->getProgramEntries($parentPath, "raw");
	
				$program["parent_path"] = substr($parentPath, 0, -1);
				$program["parent"] = $parentProgramCategories;
	
				$cache->save($cacheKey, json_encode($program), 6000);
				$program["from_cache"] = false;

//				unset($program["thumb"]);
				unset($program["parent"]);

				return [$program];
			}
			else {
				$programCategories = $this->gitModel->getProgramEntries($path);
	
				$parentProgramCategories = array("entries" => [], "TITRE" => "", "DESCRIPTION" => "", "cooked" => "");
	
				$parentPath = substr($path, 0, strlen($path) - 1);
				$parentPath = explode("/", $parentPath);
				unset($parentPath[count($parentPath) - 1]);
				$parentPath = implode("/", $parentPath) . "/";
	
				if ($parentPath && $parentPath != "/") {
					$parentProgramCategories = $this->getProgramEntries($parentPath, "raw");
				}
	
				$programCategories["parent_path"] = substr($parentPath, 0, -1);
				$programCategories["parent"] = $parentProgramCategories;
	
				$contentCache = json_encode($programCategories);
	
				$setInCache = $cache->save($cacheKey, $contentCache, 6000);
				$programCategories["from_cache"] = false;
				$programCategories["set_in_cache"] = $setInCache;
			}
		}
		else {
			$programCategories = json_decode($contentCache, true);

			if (!isset($programCategories["entries"])) {

				return [$programCategories];
			}
		}

		$programPoints = [];

		foreach($programCategories["entries"] as $entry) {
			$childrenPoints = $this->getAllProgramPoints($entry["path"] . "/");

			$programPoints = array_merge($programPoints, $childrenPoints);
		}
		
		return $programPoints;
	}

	protected function getProgramEntries($path = "", $format = "html") {
		$path = str_replace("___", "/", $path);

		$cache = \Config\Services::cache();

		$cacheKey = "program." . md5($path);
//		$cacheKey = "program";

		$contentCache = $cache->get($cacheKey);
		if (!$contentCache) {
			$programCategories = $this->gitModel->getProgramEntries($path);

			$parentProgramCategories = array("entries" => [], "TITRE" => "", "DESCRIPTION" => "", "cooked" => "");

			$parentPath = substr($path, 0, strlen($path) - 1);
			$parentPath = explode("/", $parentPath);
			unset($parentPath[count($parentPath) - 1]);
			$parentPath = implode("/", $parentPath) . "/";

			if ($parentPath && $parentPath != "/") {
				$parentProgramCategories = $this->getProgramEntries($parentPath, "raw");
			}

			$programCategories["parent_path"] = substr($parentPath, 0, -1);
			$programCategories["parent"] = $parentProgramCategories;

			$contentCache = json_encode($programCategories);

			$setInCache = $cache->save($cacheKey, $contentCache, 600);
			$programCategories["from_cache"] = false;
			$programCategories["set_in_cache"] = $setInCache;
		}
		else {
			$programCategories = json_decode($contentCache, true);
			$programCategories["from_cache"] = true;
			$programCategories["set_in_cache"] = false;
		}

		$programCategories["cache_key"] = $cacheKey;

		// Renderisation d'une liste de points programmes

		switch($format) {
			case "raw":
				return $programCategories;
				break;
			case "json":
				return $this->response->setJSON($programCategories);
				break;
			case "html":
			default:
				// load the view
				return $this->fullView('Program/programList', array("category" => $programCategories, "programEntries" => $programCategories["entries"], "path" => $path, "title" => "Parti Pirate - Programme"));
				break;
		}
	}
}