<?php

namespace App\Controllers;

use App\Models\GitModel;
use CodeIgniter\HTTP\RequestInterface;
use CodeIgniter\HTTP\ResponseInterface;
use Psr\Log\LoggerInterface;

class SearchController extends PortalController {

	/**
	 * Constructor.
	 *
	 * @param RequestInterface  $request
	 * @param ResponseInterface $response
	 * @param LoggerInterface   $logger
	 */
	public function initController(RequestInterface $request, ResponseInterface $response, LoggerInterface $logger) {
		parent::initController($request, $response, $logger);
	}
	
	protected function score($article, $search) {
		if (!isset($article["cooked"])) {
			$article["cooked"] = $this->getArticleFromCache($article["path"])["cooked"];
		}

		$scoring = [];

		$content = strtolower($article["cooked"]);

		if (isset($search["search"])) {
			$words = explode(" ", $search["search"]);

//			$scoring["words"] = $words;
			$scoring["words"] = [];

			foreach($words as $index => $word) {
				$count = substr_count($content, strtolower($word));
				$count += substr_count(strtolower($article["TITRE"]), strtolower($word));

				$scoring["words"][$word] = $count;
			}
		}

		return $scoring;
	}
	
	protected function normalizeScores($articles) {

		foreach($articles as $index => $article) {
			$articles[$index]["score"] = 0;
		}

		if (isset($articles[0]["scoring"]["words"])) {
			$maxWordScores = $this->getWordMaxScores($articles);

			foreach($articles as $index => $article) {
				$localScore = 0;
				$numberOfScores = 0;

				foreach($maxWordScores as $word => $score) {
					if ($score == 0) continue;

					$numberOfScores++;
					$localScore += $article["scoring"]["words"][$word] / $score;
				}

				if (!$numberOfScores) continue;

				$localScore /= $numberOfScores;
				$articles[$index]["score"] += $localScore;
			}
		}

		foreach($articles as $index => $article) {
			$articles[$index]["score"] = round(100 * $articles[$index]["score"]);
		}

		foreach($articles as $index => $article) {
			if ($article["score"] == 0) {
				unset($articles[$index]);
			}
		}

		return $articles;
	}

	protected function getWordMaxScores($articles) {
		$maxScores = [];

		foreach($articles as $index => $article) {
			foreach($article["scoring"]["words"] as $word => $score) {
				if (!isset($maxScores[$word]) || $maxScores[$word] < $score) {
					$maxScores[$word] = $score;
				}
			}
		}

		return $maxScores;
	}

    static function sortByScore($articleA, $articleB) {
        return $articleB["score"] <=> $articleA["score"];
    }
}