<?php

namespace App\Models;

use CodeIgniter\Model;
use Parsedown\ParsedownExtra;
use Parsedown\Parsedown;

class GitModel
{
    public $repository = null;
/*
    protected $db;

    public function __construct(ConnectionInterface &$db)
    {
        $this->db = &$db;
    }
*/

    /**
     * Get list of article metadata
     */
    function getArticles($path = "")
    {
        $dirs = $this->getEntries($path);
        $articles = array();

        foreach($dirs as $index => $dir) {
            if ($dir["type"] == "blob") {
//                unset($dirs[$index]);
            }
            else if ($dir["type"] == "tree") {
                $articleMetadata = $this->getArticleMetadata($path . $dir["file_name"]);
                if ($articleMetadata) {
                    $articles[] = $articleMetadata;
                }
            }
        }

        return $articles;
    }

    function getArticleMetadata($path) {
        $dirs = $this->getEntries($path);
        $metadata = null;

        $url = null;
        $thumbUrl = null;

        foreach($dirs as $index => $dir) {
            if ($dir["type"] == "blob" && strpos($dir["file_name"], ".json") !== false) {
                $url = str_replace("tree", "raw", $this->repository) . "/" . $path . "/" . $dir["file_name"];
/*
                $metadata["path"] = $path;
//                https://sources.partipirate.org/farlistener/articles/-/raw/master/le-travail-vision-du-futur/le-travail-vision-du-futur.json
                $metadata["url"] = str_replace("tree", "raw", $this->repository) . "/" . $path . "/" . $dir["file_name"];
                $metadata["file_name"] = $dir["file_name"];
*/
            }
            else if ($dir["type"] == "blob" && strpos($dir["file_name"], "_miniature") !== false) {
                $thumbUrl = str_replace("tree", "raw", $this->repository) . "/" . $path . "/" . $dir["file_name"];
            }
        }

        if (!$url) return null;

        $contents = file_get_contents($url);
        $metadata = json_decode($contents, true);

        if ($metadata["STATUT"] != "publié") return null;

        $metadata["path"] = $path;

        if ($thumbUrl) {
            $thumbContentType = "image/jpeg;base64";

            if (strpos($thumbUrl, ".png")) {
                $thumbContentType = "image/png;base64";
            }

            $metadata["thumb"] = "data:$thumbContentType," . base64_encode(file_get_contents($thumbUrl));
        }

        return $metadata;
    }

    /**
     * Get the article metadata and content
     */
    function getArticle($path)
    {
        $dirs = $this->getEntries($path);
        $metadata = null;

        $url = null;
        $contentUrl = null;
        $thumbUrl = null;

        foreach($dirs as $index => $dir) {
            if ($dir["type"] == "blob" && strpos($dir["file_name"], ".json") !== false) {
                $url = str_replace("tree", "raw", $this->repository) . "/" . $path . "/" . $dir["file_name"];
            }
            else if ($dir["type"] == "blob" && strpos($dir["file_name"], ".md") !== false) {
                $contentUrl = str_replace("tree", "raw", $this->repository) . "/" . $path . "/" . $dir["file_name"];
            }
            else if ($dir["type"] == "blob" && strpos($dir["file_name"], "_miniature") !== false) {
                $thumbUrl = str_replace("tree", "raw", $this->repository) . "/" . $path . "/" . $dir["file_name"];
            }
        }

        if (!$url) return null;

        $contents = file_get_contents($url);
        $metadata = json_decode($contents, true);

        if ($metadata["STATUT"] != "publié") return null;

        $metadata["path"] = $path;
        $metadata["content"] = file_get_contents($contentUrl);

        $metadata["summary"] = $this->summarize($metadata["content"]);

        $metadata["cooked"] = $this->cook($metadata["content"], $metadata["LIENS"], $metadata["summary"]);

        if ($thumbUrl) {
            $thumbContentType = "image/jpeg;base64";

            if (strpos($thumbUrl, ".png")) {
                $thumbContentType = "image/png;base64";
            }

            $metadata["thumb"] = "data:$thumbContentType," . base64_encode(file_get_contents($thumbUrl));
        }

        return $metadata;
    }

    // END OF article
    
    // program
    
    
    function getProgramEntry($path) {
        $entry = array("path" => $path, "TITRE" => $path, "thumb" => null, "DATE" => "2021-03-13", "LIENS" => array());

        $contentUrl = str_replace("tree", "raw", $this->repository) . "/" . $path . ".md";
        $content = @file_get_contents($contentUrl);

        if($content === FALSE) {
            return null;
        }

        $metadataUrl = str_replace("tree", "raw", $this->repository) . "/" . $path . ".json";
        $metadata = json_decode(@file_get_contents($metadataUrl), true);

        if ($metadata["STATUT"] != "voté") return null;

        $entry["TAGS"] = $metadata["TAGS"];
        $entry["DATE"] = $metadata["DATE"];

        $summary = $this->summarize($content);

        $entry["TITRE"] = $summary[0];
        $entry["content"] = $this->normalize($content);
        $entry["summary"] = $this->summarize($entry["content"]);

        $entry["cooked"] = $this->cook($entry["content"], $entry["LIENS"], $entry["summary"]);

        $thumbPath = substr($path, 0, strrpos($path, "/"));

        $thumbUrl = str_replace("tree", "raw", $this->repository) . "/" . $thumbPath . "/README.png";
        $thumbContentType = "image/png;base64";
        $entry["thumb"] = "data:$thumbContentType," . base64_encode(file_get_contents($thumbUrl));

        return $entry;
    }

    /**
     * Get list of program entries
     */
    function getProgramEntries($path = "") {
        $dirs = $this->getEntries($path);
        $entries = array();
        $parentEntry = array("TITRE" => "Titre par défaut", "DESCRIPTION" => "Description par défaut");

        foreach($dirs as $index => $dir) {
            if ($dir["type"] == "blob" && strpos($dir["file_name"], ".md")) {
                if (strpos($dir["file_name"], "README.md") !== false) {
                    // Contient le texte explicatif de cette arborescence
                    $contentUrl = str_replace("tree", "raw", $this->repository) . "/" . $path . "/" . $dir["file_name"];
                    $content = file_get_contents($contentUrl);
                    $summary = $this->summarize($content);
                    $parentEntry["TITRE"] = $summary[0];
                    $parentEntry["DESCRIPTION"] = $content;
                    $parentEntry["cooked"] = $this->cook($content);
                }
                else {
                    $entry = array("path" => $path . str_replace(".md", "", $dir["file_name"]), "TITRE" => $dir["file_name"], "thumb" => null, "DATE" => "2021-03-13", "LIENS" => array());
                    $contentUrl = str_replace("tree", "raw", $this->repository) . "/" . $entry["path"] . ".md";
                    $content = file_get_contents($contentUrl);
                    $summary = $this->summarize($content);
                    $entry["TITRE"] = $summary[0];

                    $thumbUrl = str_replace("tree", "raw", $this->repository) . "/" . $path . "/README.png";
                    $thumbContentType = "image/png;base64";
                    $entry["thumb"] = "data:$thumbContentType," . base64_encode(file_get_contents($thumbUrl));

                    $entries[] = $entry;
                }
            }
            else if ($dir["type"] == "tree") {
                $entry = array("path" => $path . $dir["file_name"], "TITRE" => $dir["file_name"], "thumb" => null, "DATE" => "2021-03-13", "LIENS" => array());

                $contentUrl = str_replace("tree", "raw", $this->repository) . "/" . $entry["path"] . "/README.md";
                $content = file_get_contents($contentUrl);

                $entry["content"] = $content;
                $summary = $this->summarize($content);
                $entry["TITRE"] = $summary[0];

                $thumbUrl = str_replace("tree", "raw", $this->repository) . "/" . $entry["path"] . "/README.png";
                $thumbContentType = "image/png;base64";

                $imageContent = @file_get_contents($thumbUrl);
                if ($imageContent) {
                    $entry["thumb"] = "data:$thumbContentType," . base64_encode($imageContent);
                }

                $entries[] = $entry;
            }
        }

		usort($entries, array('App\Models\GitModel', "sortByTitle"));
        $parentEntry["entries"] = $entries;

        return $parentEntry;
    }

    function normalize($content) {

        $regex = '/^# (.*)$/m';
        preg_match_all($regex, $content, $matches, PREG_SET_ORDER, 0);

        foreach($matches as $match) {
            $toReplace = "# " . $match[1];
            $content = str_replace($toReplace, "", $content);
        }

        $offset = "#";

        for($index = 2; $index < 5; $index++) {
            $previousOffset = $offset;
            $offset .= "#";
            
            $regex = '/^'.$offset.' (.*)$/m';
            preg_match_all($regex, $content, $matches, PREG_SET_ORDER, 0);
    
            foreach($matches as $match) {
                $toReplace = $previousOffset ." " . $match[1];
                $content = str_replace("#" . $toReplace, $toReplace, $content);
            }
        }

        $content = trim($content);

        return $content;
    }

    // END OF program

    function summarize($content) {
        $entries = array();

        // TODO $regex = '/^(#+) (.*)$/m';
        $regex = '/^(#) (.*)$/m';
        preg_match_all($regex, $content, $matches, PREG_SET_ORDER, 0);

        foreach($matches as $match) {
            $level = strlen($match[1]);
            $entries[] = $match[2];
        }

        return $entries;
    }

    function cook($content, $links = array(), $summary = array()) {
        error_reporting(E_ALL);
        ini_set("display_errors", 1);

        $parsedown = new ParsedownExtra();

        foreach($summary as $index => $entry) {
            $content = str_replace("# $entry", "# $entry {#summary-$index}", $content);
        }

        $content = $parsedown->text($content);

        $offset = 0;
        $sourceLinks = "";
        $toUseLinks = array();
        foreach($links as $link) {
            if ($link["type"] != "source") continue;

            $anchorPosition = strpos($content, $link["ancre"]);

            if ($anchorPosition === false) continue;

            $offset++;
            $anchorPosition += strlen($link["ancre"]);

            $link["position"] = $anchorPosition;
            $link["offset"] = $offset;
            
            $toUseLinks[] = $link;
        }

		usort($toUseLinks, array('App\Models\GitModel', "sortByPosition"));

        foreach($toUseLinks as $index => $link) {
            $toUseLinks[$index]["offset"] = $index + 1;
        }

		usort($toUseLinks, array('App\Models\GitModel', "sortByPosition"));

        foreach($toUseLinks as $index => $link) {
            $anchor = "<a href=\"#nbp-" . $link["offset"] . "\" class='nbp no-underline hover:underline' style=\"color: #9500ff\"><sup>" . $link["offset"] . "</sup></a>";

            $anchorPosition = strpos($content, $link["ancre"]);

            $anchorPosition += strlen($link["ancre"]);
/*            
            $startContent = substr($content, 0, $anchorPosition);
            $endContent = substr($content, $anchorPosition);

            $content = $startContent . $anchor . $endContent;
  */          
            $content = substr_replace($content, $anchor, $anchorPosition, 0);
        }

		usort($toUseLinks, array('App\Models\GitModel', "sortByOffset"));

        foreach($toUseLinks as $index => $link) {
            $sourceLink = "<li id=\"nbp-" . $link["offset"] . "\">" . $link["offset"] . " : <a href=\"". $link["url"] . "\" target=\"_blank\" class=\"no-underline hover:underline\">" . 
                ((isset($link["description"]) && $link["description"]) ? $link["description"] : $link["url"]) . 
                "</a></li>";
                
            $sourceLinks .= $sourceLink;
        }

        if ($sourceLinks) {
            $content .= "<hr class=\"mt-5 mb-5\">";
            $content .= "<ul id=\"sources\">";
            $content .= $sourceLinks;
            $content .= "</ul>";
        }

        $content = str_replace("<h1", "<h1 class=\"uppercase font-black text-2xl mb-2 mt-2\" ", $content);
        $content = str_replace("<h2", "<h2 class=\"uppercase font-extrabold text-xl mb-1 mt-1\" ", $content);
        $content = str_replace("<h3", "<h3 class=\"uppercase font-bold text-lg mb-1 mt-1\" ", $content);
        $content = str_replace("<h4", "<h4 class=\"uppercase font-semibold text-base mb-1 mt-1\" ", $content);
        $content = str_replace("<h5", "<h5 class=\"uppercase font-medium text-base mb-1 mt-1\" ", $content);

        $content = str_replace("« ", "«&nbsp;", $content);
        $content = str_replace(" »", "&nbsp;»", $content);
        $content = str_replace(" ?", "&nbsp;?", $content);
        $content = str_replace(" !", "&nbsp;!", $content);
        $content = str_replace("...", "…", $content);
        $content = str_replace(" …", "…", $content);

        return $content;
    }

    function getEntries($path) {
        
//        Request URL:  https://sources.partipirate.org/farlistener/articles/-/refs/master/logs_tree/?format=json&offset=0
//                      https://sources.partipirate.org/farlistener/articles/-/tree/master/logs_tree/?format=json&offset=0

        $entries = array();
        $offset = 0;

        do {
            $url = str_replace("tree", "refs", $this->repository) . "/" . $path . ($path ? "/" : "") . "logs_tree/?format=json&offset=$offset";
            $contents = file_get_contents($url);
            $contents = json_decode($contents, true);
            
            $entries = array_merge($entries, $contents);
            $offset += 25;
        }
        while(count($contents) == 25);

//        $contents["gitlab_url"] = $url;
//        print_r($entries);

        return $entries;
    }

    static function sortByDate($dataA, $dataB) {
        return $dataB["DATE"] <=> $dataA["DATE"];
    }

    static function sortByTitle($dataA, $dataB) {
        $titleA = strtolower(str_replace(array("é", "É"), array("e", "E"), $dataA["TITRE"]));
        $titleB = strtolower(str_replace(array("é", "É"), array("e", "E"), $dataB["TITRE"]));

        return $titleA <=> $titleB;
    }

    static function sortByPosition($anchorA, $anchorB) {
        if ($anchorA["position"] == $anchorB["position"]) {
            return $anchorB["offset"] - $anchorA["offset"];
        }
        
        return $anchorA["position"] - $anchorB["position"];
    }

    static function sortByOffset($anchorA, $anchorB) {
        return $anchorA["offset"] - $anchorB["offset"];
    }

    static function getSpecialPageCacheKey($path) {
        $cacheKey = "sp." . md5($path);

        return $cacheKey;
    }

    static function getArticleCacheKey($path) {
        $cacheKey = "article." . md5($path);

        return $cacheKey;
    }

    static function getProgramCacheKey($path) {
        $cacheKey = "program." . md5($path);

        return $cacheKey;
    }
}